
    # <Music score creator. Generate a sheet music from an audio.>
    # Copyright (C) <2014>  <Jose Carlos Montanez Aragon>

    # This program is free software: you can redistribute it and/or modify
    # it under the terms of the GNU General Public License as published by
    # the Free Software Foundation, either version 3 of the License, or
    # (at your option) any later version.

    # This program is distributed in the hope that it will be useful,
    # but WITHOUT ANY WARRANTY; without even the implied warranty of
    # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    # GNU General Public License for more details.

    # You should have received a copy of the GNU General Public License
    # along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import pyaudio
import wave
import time
from pylab import*
from scipy.io import wavfile
import numpy

class AudioData():
	def __init__(self):
		self.channels = 1
		self.format = pyaudio.paInt16
		self.rate = 8000
		self.frames = 0
		self.chunk = 2000
		self.record_seconds = 1
		self.sample_size = 0
		self.quarter_note_minute = 60 
		self.measure = '3/4'

# Range for each note
conversion = [(10000.0,1077.0,"r"),
			(1077.0,1017.1,"c''"),
			(1017.0,960.0,"b''"),	
			(960.0,906.2,"ais''"),
			(906.2,855.3,"a''"),
			(855.3,807.3,"gis''"),
			(807.3,762.0,"g''"),
			(762.0,719.2,"fis''"),
			(719.2,678.9,"f''"),
			(678.9,640.8,"e''"),
			(640.8,604.8,"dis''"),
			(604.8,570.8,"d''"),
			(570.8,538.8,"cis''"),
			(538.8,508.6,"c''"),
			(508.6,480.0,"b'"),
			(480.0,453.1,"ais'"),
			(453.1,427.7,"a'"),
			(427.7,403.6,"gis'"),
			(403.6,381.0,"g'"),
			(381.0,359.6,"fis'"),
			(359.6,339.4,"f'"),
			(339.4,320.4,"e'"),
			(320.4,302.4,"dis'"),
			(302.4,285.4,"d'"),
			(285.4,269.4,"cis'"),
			(269.4,254.3,"c'"),
			(254.3,240.0,"b"),
			(240.0,226.5,"ais"),
			(226.5,213.8,"a"),
			(213.8,201.8,"gis"),
			(201.8,190.5,"g"),
			(190.5,179.8,"fis"),
			(179.8,169.7,"f"),
			(169.7,160.2,"e"),
			(160.2,0.0,"r")]


def tempo(audioData):
	quarter_note = audioData.quarter_note_minute/60.0 
	semiquaver = 4*quarter_note

	# Determining the size of the chunk. 
	# That is, the number of semiquaver for second
	audioData.chunk = int(ceil(2530 - 200*semiquaver))

	return audioData



def record(audioData):
	global sounddirectory
	p = pyaudio.PyAudio()

	stream = p.open(format=audioData.format,
	                channels=audioData.channels,
	                rate=audioData.rate,
	                input=True,
	                frames_per_buffer=audioData.chunk) #buffer

	print("* recording")

	frames = []

	# Record the audio
	for i in range(0, int(audioData.rate / audioData.chunk * audioData.record_seconds)):
	    data = stream.read(audioData.chunk)
	    frames.append(data) # 2 bytes(16 bits) per channel

	print("* done recording")

	# Setup some data
	audioData.sample_size = p.get_sample_size(audioData.format)
	stream.stop_stream()

	# The audio is saved to work with it later
	# We save in /tmp 
	sounddirectory = "/tmp/"+time.strftime("%H%M%S")+".wav"
	wf = wave.open(sounddirectory, 'wb')
	wf.setnchannels(audioData.channels)
	wf.setsampwidth(p.get_sample_size(audioData.format))
	audioData.sample_size = p.get_sample_size(audioData.format)
	wf.setframerate(audioData.rate)
	wf.writeframes(b''.join(frames))
	wf.close()

	stream.close()
	p.terminate()
	return (stream, frames, sounddirectory)

def save(dirname, filename, soundfile, audioData):
	wf = wave.open(dirname+"/"+filename+".wav", 'wb')
	wf.setnchannels(audioData.channels)
	wf.setsampwidth(audioData.sample_size)
	wf.setframerate(audioData.rate)
	wf.writeframes(b''.join(audioData.frames))
	wf.close()

def play(soundfile, aData, sounddirectory):

	f = wave.open(sounddirectory,"rb")  
	p = pyaudio.PyAudio()  

	stream = p.open(format = p.get_format_from_width(f.getsampwidth()),  
	                channels = f.getnchannels(),  
	                rate = f.getframerate(),  
	                output = True)  
	data = f.readframes(aData.chunk)  

	while data != '':  
	    stream.write(data)  
	    data = f.readframes(aData.chunk)  
  
	stream.stop_stream()  
	stream.close()   
	p.terminate()  

def audioProcessing(filename, audioData):
	list_of_freq = getListofFreq(filename, audioData)
	final_list = preprocessingFreqs(list_of_freq, audioData)
	writeFile(final_list, audioData)
	os.system("lilypond -s score.ly")


def getListofFreq(filename, audioData):
	wf = wave.open(filename, 'rb')
	swidth = wf.getsampwidth()
	RATE = wf.getframerate()
	audioData = tempo(audioData)

	# Use a Blackman window
	window = numpy.blackman(audioData.chunk)

	# Read some data
	data = wf.readframes(audioData.chunk)
	# Find the frequency of each chunk
	list_of_freq = []
	while len(data) == audioData.chunk*swidth:

	    # Unpack the data and times by the hamming window
	    indata = numpy.array(wave.struct.unpack("%dh"%(len(data)/swidth),\
	                                         data))*window
	    # Take the fft and square each value
	    fftData=abs(numpy.fft.rfft(indata))**2
	    # Find the maximum
	    which = fftData[1:].argmax() + 1
	    # Use quadratic interpolation around the max
	    if which != len(fftData)-1:
	        y0,y1,y2 = numpy.log(fftData[which-1:which+2:])
	        x1 = (y2 - y0) * .5 / (2 * y1 - y2 - y0)
	        # Find the frequency and output it
	        thefreq = (which+x1)*RATE/audioData.chunk
	        list_of_freq.append(thefreq)
	    else:
	        thefreq = which*RATE/audioData.chunk
	        list_of_freq.append(thefreq)
	    # Read some more data
	    data = wf.readframes(audioData.chunk)

	return list_of_freq

def preprocessingFreqs(list_of_freq, audioData):

	# Transform the original freq to the note
	list_ = []
	for i in list_of_freq:
		for j in conversion:
			if j[1] < i <= j[0]:
				list_.append(j[2])
	
	# Calculate the number of pieces of audio per measure
	n = 0
	if audioData.measure == '2/4':
		n = 2*int(round(audioData.quarter_note_minute*0.0667))
	if audioData.measure == '3/4':
		n = 3*int(round(audioData.quarter_note_minute*0.0667))
	if audioData.measure == '4/4':
		n = 4*int(round(audioData.quarter_note_minute*0.0667))

	# From the before list, we obtain the number of repetitions of
	# a same note
	final_list = []
	j = 0
	for m in range(len(list_)/n + 1):
		note = list_[j] 
		cont = 64
		for i in range(1,n):
			if j+i <= len(list_)-1:
				if list_[j+i] == note:
					cont = cont / 2
				else:
					final_list.append(note + str(cont/2))
					cont = 64
					note = list_[j+i]
					if i+j+1 == len(list_):
						final_list.append(note + str(cont/2))
		
		j = j + n

	return final_list

def writeFile(final_list, audioData):
	f = open('score.ly', "w")

	f.write("\score {\n\t\\version \"2.16.2\"{\n\t\t\\time 4/4\n\t\t\key c \major\n\t\t\\tempo 4 = ")
	f.write(str(audioData.quarter_note_minute) + "\n\t\t")
	for i in (final_list):
		f.write(i)
		f.write(" ")
	f.write("\\bar \"|.\"\n\t}\n}\n")

	f.close()



#!/usr/bin/env python

    # <Music score creator. Generate a sheet music from an audio.>
    # Copyright (C) <2014>  <Jose Carlos Montanez Aragon>

    # This program is free software: you can redistribute it and/or modify
    # it under the terms of the GNU General Public License as published by
    # the Free Software Foundation, either version 3 of the License, or
    # (at your option) any later version.

    # This program is distributed in the hope that it will be useful,
    # but WITHOUT ANY WARRANTY; without even the implied warranty of
    # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    # GNU General Public License for more details.

    # You should have received a copy of the GNU General Public License
    # along with this program.  If not, see <http://www.gnu.org/licenses/>.

# -*- coding: utf-8 -*-

import os
import wx
import time
import music_score_creator
from music_score_creator.sound import *

def initialize():
    # Create all the variables.
    global audioData
    audioData = AudioData()


class MainWindow(wx.Frame):
    def __init__(self, parent, title):
        wx.Frame.__init__(self, parent, style=wx.DEFAULT_FRAME_STYLE ^ wx.RESIZE_BORDER, title=title, size=(300,300))
        initialize()
        global menuSave, menuPlay, soundfile, audioData, menuAudioProcess, sounddirectory, tRecord
        panel = wx.Panel(self)

        # Setting up the menu.
        filemenu = wx.Menu()
        soundmenu = wx.Menu()
        helpmenu = wx.Menu()

        # Creating the items in the menu
        # --- File menu ---
        menuOpen = filemenu.Append(wx.ID_OPEN, "&Open", "Open a file")
        menuSave = filemenu.Append(wx.ID_SAVE, "Save", "Save file")
        menuExit = filemenu.Append(wx.ID_EXIT,"E&xit"," Terminate the program")
        menuSave.Enable(False)


        # --- Sound menu ---
        menuRecord = soundmenu.Append(wx.ID_ANY, "&Record", "Record an audio")
        menuPlay = soundmenu.Append(wx.ID_ANY, "Play", "Play an audio")
        menuAudioProcess = soundmenu.Append(wx.ID_ANY, "Generate PDF", "Generate PDF from audio")
        menuPlay.Enable(False)
        menuAudioProcess.Enable(False)

        # --- Help menu ---
        menuAbout = helpmenu.Append(wx.ID_ABOUT, "&About"," Information about this program")
        

        # Creating the menubar.
        menuBar = wx.MenuBar()
        menuBar.Append(filemenu,"&File") 
        menuBar.Append(soundmenu, "&Sound")
        menuBar.Append(helpmenu, "&Help")
        self.SetMenuBar(menuBar)  # Adding the MenuBar to the Frame content.

        # Set events of the menubar.
        self.Bind(wx.EVT_MENU, self.OnAbout, menuAbout)
        self.Bind(wx.EVT_MENU, self.OnExit, menuExit)
        self.Bind(wx.EVT_MENU, self.OnRecord, menuRecord)
        self.Bind(wx.EVT_MENU, self.OnPlay, menuPlay)
        self.Bind(wx.EVT_MENU, self.OnAudioProcess, menuAudioProcess)
        self.Bind(wx.EVT_MENU, self.OnOpen, menuOpen)
        self.Bind(wx.EVT_MENU, self.OnSave, menuSave)


        # Creating the toolbar
        vbox = wx.BoxSizer(wx.VERTICAL)
        global mainToolbar
        mainToolbar = wx.ToolBar(self)
        toolbarOpen = mainToolbar.AddLabelTool(wx.ID_OPEN, '', wx.Bitmap(music_score_creator.__path__[0] + '/images/open32.png'))
        toolbarSave = mainToolbar.AddLabelTool(wx.ID_SAVE, '', wx.Bitmap(music_score_creator.__path__[0] + '/images/save32.png'))
        mainToolbar.AddSeparator()
        toolbarRecord = mainToolbar.AddLabelTool(5990, '', wx.Bitmap(music_score_creator.__path__[0] + '/images/record32.png'))
        toolbarPlay = mainToolbar.AddLabelTool(6001, '', wx.Bitmap(music_score_creator.__path__[0] + '/images/play32.png'))
        toolbarAudioProcess = mainToolbar.AddLabelTool(6002, '', wx.Bitmap(music_score_creator.__path__[0] + '/images/pdf32.png'))
        mainToolbar.EnableTool(wx.ID_SAVE, False) # Before we have an audio, it's deactivated.
        mainToolbar.EnableTool(6001, False)
        mainToolbar.EnableTool(6002, False)
        mainToolbar.Realize()

        # Set events of the toolbar

        self.Bind(wx.EVT_MENU, self.OnOpen, toolbarOpen)
        self.Bind(wx.EVT_MENU, self.OnRecord, toolbarRecord)
        self.Bind(wx.EVT_MENU, self.OnPlay, toolbarPlay)
        self.Bind(wx.EVT_MENU, self.OnAudioProcess, toolbarAudioProcess)
        self.Bind(wx.EVT_MENU, self.OnSave, toolbarSave)

        vbox.Add(mainToolbar, 0, wx.EXPAND)

        # 
        tempos = ['60', '90', '120', '150']
        wx.StaticText(self, label=("Tempo"), pos=(10, 74))
        cb_tempo = wx.ComboBox(self, value=('60'), pos=(209, 70), size=(80, 28), choices=tempos, 
            style=wx.CB_READONLY)

        wx.StaticLine(self, pos=(0, 50), size=(300,1))

        tRecord = wx.TextCtrl(self,-1, pos=(209, 110), value="2")
        wx.StaticText(self, label=("Recording time (seconds)"), pos=(10, 114))
        #wx.StaticText(self, label=("seconds"), pos=(210, 114))

        measures = ['2/4', '3/4', '4/4']
        wx.StaticText(self, label=("Measure"), pos=(10, 154))
        cb_measure = wx.ComboBox(self, value=('4/4'), pos=(209, 150), size=(80, 28), choices=measures, 
            style=wx.CB_READONLY)


        # Events of the block

        self.Bind(wx.EVT_COMBOBOX, self.OnTempo, cb_tempo)
        self.Bind(wx.EVT_COMBOBOX, self.OnMeasure, cb_measure)

        self.SetSizer(vbox)

        self.Show(True)

    def OnOpen(self, e):
        global soundfile, audioData, sounddirectory
        self.dirname = '.'
        dlg = wx.FileDialog(self, "Choose a file", self.dirname, "", "*.wav", wx.OPEN)
        if dlg.ShowModal() == wx.ID_OK:
            self.filename = dlg.GetFilename()
            self.dirname = dlg.GetDirectory()
            sounddirectory = self.dirname+"/"+self.filename
            soundfile = wave.open(self.dirname+"/"+self.filename, 'rb')
            mainToolbar.EnableTool(wx.ID_SAVE, True)
            mainToolbar.EnableTool(6001, True)
            mainToolbar.EnableTool(6002, True)
            menuSave.Enable(True)
            menuPlay.Enable(True)
            menuAudioProcess.Enable(True) 
            
        dlg.Destroy()

    def OnRecord(self, e):
        global soundfile,menuSave, menuPlay, menuAudioProcess, audioData, sounddirectory
        global tRecord
        mainToolbar.EnableTool(5990, False)
        time.sleep(3)
        audioData.record_seconds = int(tRecord.GetValue())
        (soundfile, frames, sounddirectory) = record(audioData)
        audioData.frames = frames
        mainToolbar.EnableTool(5990, True)
        mainToolbar.EnableTool(wx.ID_SAVE, True)
        mainToolbar.EnableTool(6001, True)
        mainToolbar.EnableTool(6002, True)
        menuSave.Enable(True)
        menuPlay.Enable(True)
        menuAudioProcess.Enable(True)

    def OnAudioProcess(self, e):
        #Add interaction to save the file. In test, only the path to the file to process
        global sounddirectory
        audioProcessing(sounddirectory, audioData)

    def OnSave(self, e):
        global soundfile, audioData
        self.dirname = '.'
        dlg = wx.FileDialog(self, "Save audio", self.dirname, "", "*.wav", wx.SAVE)
        if dlg.ShowModal() == wx.ID_OK:
            self.filename = dlg.GetFilename()
            self.dirname = dlg.GetDirectory()
            save(self.dirname, self.filename, soundfile, audioData)
        dlg.Destroy()

    def OnAbout(self, e):
        dlg = wx.MessageDialog( self, "An attempt of doing a music score creator.\nVersion 0.6beta - 2014\nCreated by Jose Carlos M. Aragon.\nYou can contact me via twitter: @Montagon.", "About Music score creator", wx.OK)
        dlg.ShowModal()
        dlg.Destroy()

    def OnPlay(self, e):
        global soundfile, audioData, sounddirectory
        play(soundfile, audioData, sounddirectory)

    def OnExit(self, e):
        self.Close(True)

    def OnTempo(self, e):
        global audioData
        audioData.quarter_note_minute = int(e.GetString())

    def OnMeasure(self, e):
        global audioData
        audioData.measure = e.GetString()

app = wx.App(False)
frame = MainWindow(None, "Music score creator")
app.MainLoop()


import sys

from setuptools import setup, find_packages

setup(
    name='music-score-creator',
    version='0.6',
    description="Music score creator. Generate a music score from an audio",
    long_description=open('README.srt').read(),
    author = 'Jose Carlos Montanez Aragon',
    author_email = 'jose90clari@gmail.com',
    license='GPLv3',
    packages=['music_score_creator'],
    data_files=[('music_score_creator/images', ['music_score_creator/images/open32.png', 'music_score_creator/images/pdf32.png', 'music_score_creator/images/play32.png', 'music_score_creator/images/record32.png', 'music_score_creator/images/save32.png']),
                ('music_score_creator', ['music_score_creator/sound.py'])],
    scripts=['music_score_creator/music-score-creator.py'],
    url='https://github.com/Montagon/A-music_score_creator',
    download_url = 'https://github.com/Montagon/A-music_score_creator/releases/tag/v0.6-beta',
    include_package_data=True,
    install_requires=[
       'pyaudio',
       'numpy',
       'scipy',
       'matplotlib'
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: X11 Applications',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: POSIX',
        'Topic :: Multimedia :: Sound/Audio :: Capture/Recording',
        'Topic :: Scientific/Engineering',
        'Topic :: Utilities'
        ],
    keywords = ['music', 'creator', 'music score', 'audio'],
)